/***********************************************************
 * Copyright (c) 2006 - RogueSheep Incorporated
 * All Rights Reserved
 *
 * Description:
 *
 * JavaScript that checks the current document for any text frames that have overset text. 
 * User is warned if overset text is found and given a chance to cancel the current action.
 * All overset items are marked with an underlay red item on a new layer.
 *
 * Associate before any action when you want to check for overset text.
 *
 * Version: 1.0
 *
 * Works with InDesign : CS 1.0 (v3.x) or CS 2.0 (v4.0)
 *
 ***********************************************************/

gOversetMarkersLayerName = "RS Overset Markers";
gOversetMarkerColorName  = "RS Overset Marker";
gOversetMarkerColor		 = [255, 0, 0];

//
// Call the main function. In Javascript this is superfluous because
// the code can be in the global space -- this is done only for code 
// organization.  RSCheckForOversetText represents the only global line executed when
// the script is processed.
//

RSCheckForOversetText();

//
// RSCheckForOversetText iterates over all the text frames in the
// document and marks those that are overset
//

function RSCheckForOversetText()
{
	//
	// Clean up the old overset markers layer and color
	//
	
	RSRemoveOversetMarkersLayer ( app.activeDocument );
	RSRemoveOversetMarkerColor ( app.activeDocument );

	var hasOversetText = false;

	//
	// Iterate over the frames of the document
	//
	
	with ( app.activeDocument ) 
	{
		for( frameIndex = 0; frameIndex < textFrames.length; frameIndex++) 
		{
			var myTextFrame = textFrames.item( frameIndex );
			
			if ( myTextFrame.overflows ) 
			{
				RSMarkOversetItem ( app.activeDocument, myTextFrame );
				
				hasOversetText = true;
			}
	
		}	
	
	}
	
	//
	// If any text frame had overset tell the user here and give them a chance to abort
	//
	
	if ( hasOversetText ) 
	{
		var theResult = confirm ( "RSCheckForOversetText\n" +
								  "One or more text frames has overset text. Each overset frame is marked with a transparent " +
				  				  "underlay item. The markers are on a new layer named \"RS Overset Markers\".\n\n" +
				  				  "Continue action?\n\nClick Yes to continue the action and No to abort the action.", 
				  				  true);
				  		
		if ( theResult == false ) 
		{
			app.eventscriptSuppressAction();
		}
	}
}

//
// Creates a marker page item the size of the overset item.
// The marker item is filled with a transparent red and placed on
// the Overset Markers layer which is below all the other layers.
// The user can then quickly find visually all items that do not
// meet the validation criteria.
//

function RSMarkOversetItem ( document, oversetPageItem )
{
	var oversetMarkersLayer = RSGetOversetMarkersLayer ( document );
	var oversetMarkerColor  = RSGetOversetMarkerColor ( document );
	
	//
	// We need the page of this text frame so we can create a new item
	// on the same page
	//
	
	var oversetItemPage = RSGetPageOfItem ( oversetPageItem );
	
	//
	// Create the marker item on the Overset Markers layer on this page
	//
	
	if ( oversetItemPage != null )
	{
		var markerItem = oversetItemPage.polygons.add ( oversetMarkersLayer );

		//
		// There isn't an easy way to set the paths when creating the polygon.
		// Instead we set the path after creating the marker item.  This ensures
		// the marker is the same shape as the overset item.  The marker item
		// is created with a single path hence the check for null
		//
		
		var numOversetItemPaths = oversetPageItem.paths.length;
		
		for ( var n = 0; n < numOversetItemPaths; n++ )
		{
			var markerItemPath = markerItem.paths.item ( n );
			
			if ( markerItemPath == null )
			{
				markerItemPath = markerItem.paths.add();
			}
			
			markerItemPath.properties = oversetPageItem.paths.item ( n ).properties;
		}

		//
		// Set the properties here. In CS2 you can pass these successfully to the add method above
		// when adding the polygon. But in CS these properties are ignored when passed to the
		// add method -- reason unknown.
		//
		
		markerItem.properties = {   fillColor: oversetMarkerColor,
								  strokeColor: oversetMarkerColor,
								 strokeWeight: 10.0,
									  opacity: 25.0,
								  nonprinting: true   };
								  
	}
}

//
// Returns the page for the given item or null if no page (error)
//

function RSGetPageOfItem ( item )
{
	var itemParent = item.parent;
	
	while ( itemParent != "[object Document]" )
	{
		if ( itemParent == "[object Page]" )
		{
			return itemParent;
		}
		else if ( itemParent == "[object Character]" )
		{	
			if ( itemParent.parentTextFrames.length > 0 )
			{
				itemParent = itemParent.parentTextFrames[0];
			}
		}
		else
		{
			itemParent = itemParent.parent;
		}
	}
	
	return null;
}

//
// Returns the Overset Markers layer, creating it if necessary
//

function RSGetOversetMarkersLayer ( document )
{
	var oversetMarkersLayer = document.layers.item ( gOversetMarkersLayerName );
	
	// 
	// if it doesn't exist, create it
	//
	
	if ( oversetMarkersLayer == null )
	{
		oversetMarkersLayer = document.layers.add ( { name: gOversetMarkersLayerName, layerColor: gOversetMarkerColor } );
		
		oversetMarkersLayer.move ( LocationOptions.atEnd );
	}
	
	return oversetMarkersLayer;
}

//
// Returns the color we use to fill and stroke the overset markers, 
// creating the color if necessary
//

function RSGetOversetMarkerColor ( document )
{
	//
	// Make sure our overset mark swatch exists
	//
	
	var oversetMarkerColor = document.colors.item ( gOversetMarkerColorName );
	
	if ( oversetMarkerColor == null )
	{
		oversetMarkerColor = document.colors.add 
				( 
					{ 
						      name: gOversetMarkerColorName, 
						     model: ColorModel.process, 
						     space: ColorSpace.rgb,
						colorValue: gOversetMarkerColor
					}
				);
	}		
	
	return oversetMarkerColor;
}

//
// Removes the Overset Markers layer if it exists
//

function RSRemoveOversetMarkersLayer ( document )
{
	var oversetMarkersLayer = document.layers.item ( gOversetMarkersLayerName );
	
	if ( oversetMarkersLayer != null )
	{
		oversetMarkersLayer.remove();
	}
}

//
// Removes the Overset Marker color if it exists
//

function RSRemoveOversetMarkerColor ( document )
{
	var oversetMarkerColor = document.colors.item ( "RS Overset Marker" );
	
	if ( oversetMarkerColor != null )
	{
		oversetMarkerColor.remove();
	}
}

